@@ -1,13 +1,25 @@
-from transpara.logging_config import Locals, GLOBAL_VERBOSE, TRANSPARA_DEBUG_LEVEL, TRANSPARA_ERROR_LEVEL
-
+import asyncio
+import logging
 import os
 import sys
 import time
-import logging
 import traceback
+from functools import wraps
 
 import jsonpickle
 
+from transpara.logging_config import (GLOBAL_VERBOSE, TRANSPARA_DEBUG_LEVEL,
+                                      TRANSPARA_ERROR_LEVEL, Locals)
+
+
+#Async helper for wrapping decorators and prevent blocking on methods that are indeed async
+async def async_wrapper(func, *args, **kwargs):
+    if asyncio.iscoroutinefunction(func):
+        #print(f"this function is a coroutine: {func.__name__}")
+        return await func(*args, **kwargs)
+    else:
+        #print(f"not a coroutine: {func.__name__}")
+        return func(*args, **kwargs)
 
 exc_message = "RETURNEXCEPTION" #Used for returning the exception string when suppressing exceptions
 """
@@ -29,7 +41,9 @@ def log_middleware(verbose_exc=False, suppress_exc=False,
             ):
 
     def decorator(function):
-        def wrapper(*args, **kwargs):
+
+        @wraps(function)
+        async def wrapper(*args, **kwargs):
 
             if debug_elapsed:
                 start = time.time()
@@ -50,7 +64,7 @@ def log_middleware(verbose_exc=False, suppress_exc=False,
                 #set local tracer
                 if debug_locals: sys.setprofile(tracer)
                 #invoke actual function
-                result = function(*args, **kwargs)
+                result = await async_wrapper(function, *args, **kwargs)
                 #Set back original tracer
                 if debug_locals: sys.setprofile(None)        
             except BaseException as e:
@@ -111,10 +125,11 @@ def log_middleware(verbose_exc=False, suppress_exc=False,
 
 
 def debug_elapsed(func):
-    def wrapper(*args, **kwargs):
+    @wraps(func)
+    async def wrapper(*args, **kwargs):
         start = time.time()
         logger = logging.getLogger(func.__module__)
-        result = func(*args, **kwargs)
+        result = await async_wrapper(func,*args, **kwargs)
         message = f"Elapsed: {(time.time() - start)*1000}ms"
         logger.log(TRANSPARA_DEBUG_LEVEL,f"{func.__name__}(): {message}")
         return result
